/*
   Copyright (C) 1999 PolyWog and Javaman for Ghetto.Org
   This file is the command line component of the PCR1000 Control Suite.

   The PCR-1000 Control Suite is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The PCR-1000 Control Suite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the PCR-1000 API Library; see the file LICENSE.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

/* 
 * This file declars the PCR-1000 command line interface for the
 * Ghetto.org radio toolkit.  It implements as much of the functionality
 * as possible as defined in the API
 *
 */

#include <pcp.h>
#include <ctype.h>

#if defined (SunOS) || defined (Irix)
#include <strings.h>
#else 
#include <string.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

pcrfreq_t freqconv(char *);
pcrfreq_t stepconv(char *);
const char *bargraph(pcrfreq_t, pcrfreq_t, pcrfreq_t);
void cmdhelp(void);

int main (int argc, char **argv)
{
	bool scanmode = false;	// Turn on scanner mode if set.
	bool agc = false;  // Store setting of AGC after parsing and before transmission.
	bool nb = false;  	// Same as above, but for noise blanker.
	bool rf = false;   	// Same as above, but for RF attenuation.
	char *hz, *frac_hz;	// Pointers used in the parsing of the frequency.
	char upperword[4]; 	// Variable for converting lowercase to uppercase.
	char meter[81];     // Array to carry output meter.
	char c[128];        // Temp variable for a read.
	char freq[20];      // Temp variable for formatting the frequency.
	int vol = 70;      	// Same as for AGC, but for volume.
	int sql = 40;      	// Same as for AGC, but for squelch.
	int filnum = 15;   	// Same as for AGC, but for the filter IF statements.
	int i;             	// Loop counter, used multiple times.
	int hpoint = 80;	// Hold point for the scanner.
	float tonesq = 0;  	// Same as for AGC, but for the tone squelch.
    pcrfreq_t lwrfreq;  // unsigned long int lower frequency.
    pcrfreq_t uprfreq;  // unsigned long int upper frequency.
    pcrfreq_t curfreq;  // unsigned long int current frequency.
    pcrfreq_t step;     // unsigned long int step size.
	pcrcmd_t filter[11] = PCRFLTR15; // Same as for AGC, but for IF filter.
	pcrcmd_t mod[10];                // Same as for AGC, but for mod format.
	pcrcmd_t port[128] = PCRDEV;	 // Port setting for the pcr1000.

	bzero(mod, sizeof(mod));

	/*
	 * If the command is issued with no arguments, issue instructions.
	 * Otherwise, instantiate a new class, parse data, and issue commands.
	 */

	if (argc > 1) {
		/*
		 * My ghetto-assed command line parameter parsing loop.
		 * I am too lazy to learn how to use getopt(3), so this
		 * will have to do for now.  The limitation is that the
		 * value has to be attached to the parameter: -v70 for example.
		 */
		 
		while ((argc > 1) && (argv[1][0] == '-')) {
			switch ((char)tolower(argv[1][1])) {
				// Case statement for each parameter.
				case 'v':
					vol = atoi(&argv[1][2]);
					if (vol > 99 || vol < 0) {
						vol = 70;
						fprintf(stderr, "Invalid volume setting.  Volume set to %i.\n", vol);
					}
					break;

				case 's':
					scanmode = true;
					break;

				case 'a':
					agc = true;
					break;
				
				case 'n':
					nb = true;
					break;

				case 'r':
					rf = true;
					break;
                case 'h':
					hpoint = atoi(&argv[1][2]);
					if (hpoint > 255 || hpoint < 0) {
						hpoint = 80;
						fprintf(stderr, "Invalid halt point setting.  Halt point set to %i.\n", hpoint);
					}
					break;

				case 't':
					hz = &argv[1][2];
					frac_hz = strchr(&argv[1][2], '.');
		
                	tonesq = (float) atoi(hz);

					if (frac_hz == NULL) {
						break;
					} else {
        				*frac_hz = '\0';
        				frac_hz++;
						tonesq += ((float) atoi(frac_hz))/10;
						break;
					}

				case 'q':
					sql = atoi(&argv[1][2]);
					if (sql > 99 || vol < 0) {
						sql = 40;
						fprintf(stderr, "Invalid squelch setting.  Squelch set to %i.\n", sql);
					}	
					break;

				/*
				 * The only possible filter options are 3, 6, 15, 50, and 230
				 * kHz.  The user enters these values in the command line.
				 * Otherwise, the code defaults to 15 kHz.
				 */
				case 'f':
					filnum = atoi(&argv[1][2]);

					if ((filnum == 3) || (filnum == 6) || (filnum == 15) || (filnum == 50) || (filnum == 230)) {
						strcpy(filter, &argv[1][2]);
					} else {
						strcpy(filter, "15");
						fprintf(stderr, "Invalid filter setting.  IF Filter set to 15 kHz.\n");
					}
					break;
				
				case 'p':
					strcpy(port, &argv[1][2]);
					break;
					
				case 'm':
				/*
				 * The first part of the following code coverts
				 * the command lime parameter to an uppercase statement.
				 * The rest determines what the actual mode passed was.
				 */
					for (i = 2; argv[1][i] != '\0'; i++) {
						upperword[i-2] = (char) toupper(argv[1][i]);
					}

					upperword[i-2] = '\0';
					
					if (strncmp(upperword, "LSB", 3) == 0) {
						strcpy(mod, "LSB");
					} else if (strncmp(upperword, "USB", 3) == 0) {
						strcpy(mod, "USB");
					} else if (strncmp(upperword, "AM", 2) == 0) {
						strcpy(mod, "AM");
					} else if (strncmp(upperword, "CW", 2) == 0) {
						strcpy(mod, "CW");
					} else if (strncmp(upperword, "UNK", 3) == 0) {
						strcpy(mod, "UNK");
					} else if (strncmp(upperword, "NFM", 3) == 0) {
						strcpy(mod, "NFM");
					} else if (strncmp(upperword, "WFM", 3) == 0) {
						strcpy(mod, "WFM");
					} else {
						fprintf(stderr, "Invalid mode found.  Using NFM by default.\n");
						strcpy(mod, "NFM");
					}
					break;
			}
			++argv;
			--argc;
		}

		/*
		 * The folowing is to parse the frequency out into MHz and kHz
		 * components.  a pointer is places where the decimal is
		 * and all data from that point forward is considered to be
		 * the kHz component.
		 */

		if (argc == 2 && !scanmode) {
			curfreq = freqconv(argv[1]);
			if (curfreq < 50000 || curfreq > 1300000000) {
				fprintf(stderr, "Error: Cannot process frequency.\n\n");
				cmdhelp();
				exit(-1);
			}
		}

		if (argc == 4 && scanmode) {

            lwrfreq = freqconv(argv[1]);
            uprfreq = freqconv(argv[2]);
            step = stepconv(argv[3]);

			if (lwrfreq < 50000 || lwrfreq > 1300000000 || uprfreq < 50000 || uprfreq > 1300000000) {
				fprintf(stderr, "Error: Cannot process frequencies.\n\n");
				cmdhelp();
				exit(-1);
			}
		}
		
		// Pass all the commands to the radio.
		
		PCP *rig = new PCP (port, BAUDRATE, "RIG"); 

		if(!rig->PCPQueryOn()) {
			if(rig->PCPInit()) {
				fprintf(stderr, "Error: Cannot initialize radio.  Check port.\n\n");
				cmdhelp();
				exit(-1);
			}
			//Start up the radio.
			rig->PCPPowerUp();
		}

		if(!rig->PCPSetVolume(vol)) {
			fprintf(stderr, "Error: Cannot set volume, exiting.\n");
			cmdhelp();
			exit(-1);
		}
		if(!rig->PCPSetSquelch(sql)) {
			fprintf(stderr, "Error: Cannot set squelch, exiting.\n");
			cmdhelp();
			exit(-1);
		}
		if(!rig->PCPSetToneSq(tonesq)) {
			fprintf(stderr, "Error: Cannot set tone squelch, exiting.\n");
			cmdhelp();
			exit(-1);
		}
		if(!rig->PCPSetAutoGain(agc)) {
			fprintf(stderr, "Error: Cannot set AGC, exiting.\n");
			cmdhelp();
			exit(-1);
		}
		if(!rig->PCPSetNB(nb)) {
			fprintf(stderr, "Error: Cannot set NB, exiting.\n");
			cmdhelp();
			exit(-1);
		}
		if(!rig->PCPSetRFAttenuator(rf)) {
			fprintf(stderr, "Error: Cannot set RF Attenuator, exiting.\n");
			cmdhelp();
			exit(-1);
		}
		if(!scanmode) {
			if(!rig->PCPSetFreq(curfreq)) {
				fprintf(stderr, "Error: Cannot set frequency, exiting.\n");
				cmdhelp();
				exit(-1);
			}
			if(!rig->PCPSetMode(mod)) {
				fprintf(stderr, "Error: Cannot set mode, exiting.\n");
				cmdhelp();
				exit(-1);
			}
			if(!rig->PCPSetFilter(filter)) {
				fprintf(stderr, "Error: Cannot set filter, exiting.\n");
				cmdhelp();
				exit(-1);
			}

			printf("Command successful.\nFrequency: %i.%06.0i MHz; Signal Strength: %i.\n", curfreq/1000000, curfreq - ((int)(curfreq/1000000))*1000000, rig->PCPSigStrength());
			return(0);
		}
		
		if(scanmode) {
			if(!rig->PCPSetFreq(lwrfreq)) {
				fprintf(stderr, "Error: Cannot set initial frequency, exiting.\n");
				cmdhelp();
				exit(-1);
			}
			if(!rig->PCPSetMode(mod)) {
				fprintf(stderr, "Error: Cannot set mode, exiting.\n");
				cmdhelp();
				exit(-1);
			}
			if(!rig->PCPSetFilter(filter)) {
				fprintf(stderr, "Error: Cannot set filter, exiting.\n");
				cmdhelp();
				exit(-1);
			}
        
			printf("Beginning scan...\n");

		    if(fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK) < 0){
				fprintf(stderr, "Unable to set stdin to nonblocking\n");
				exit(1);
			}

			while (1) {
				for (curfreq = lwrfreq; curfreq <= uprfreq; curfreq += step) {
					if(!rig->PCPSetFreq(curfreq)) {
						fprintf(stderr, "Error: Cannot set current frequency %i hz, exiting.\n", curfreq);
						exit(-1);
					}
					strcpy(meter, "Scanning... ");
					strcat(meter, bargraph(lwrfreq, uprfreq, curfreq));
					printf("%s\r", meter);
					fflush(stdout);
					if(rig->PCPSigStrength() > hpoint) {
						while (1) {
							printf("Hit......   Press ENTER to contine.         %i.%06.0i MHz\r", curfreq/1000000, curfreq - ((int)(curfreq/1000000))*1000000);
							fflush(stdout);
							if(read(STDIN_FILENO, c, 128) < 0) {
								if(errno == EAGAIN) {
									continue;
								} else {
									fprintf(stderr, "Error in read(), exiting...\n\n");
									exit(1);
								}
							} else {
								break;
							}
							sleep(1);
						}
					}
				}
			}
		}
	} else {
		cmdhelp();
	}
}

void cmdhelp(void)
{
		printf("\npcr version 0.9.1 written by Polywog and Javaman.\n");
		printf("Usage is:\n");
		printf("\tpcr [-p<port>] [-v<volume>] [-q<squelch>] [-f<filter>] [-m<mode>] [-t<freq>] [-a] [-n] [-r] freq (in MHz)\n\n");
		printf("\tpcr [-s] [-p<port>] [-v<volume>] [-q<squelch>] [-f<filter>] [-m<mode>] [-t<freq>] [-a] [-n] [-r] -h<hold> lower_freq upper_freq (in MHz) step_freq (in kHz)\n\n");
		printf("Options:\n");
		printf("   -p\t= Set port (/dev/pcr1000 by default)\n");
		printf("   -v\t= Set volume (00 through 99)\n");
		printf("   -q\t= Set squelch (00 through 99)\n");
		printf("   -f\t= Set filter (3, 6, 15, 50, 230)\n");
		printf("   -m\t= Set mode (usb, lsb, am, cw, nfm, wfm)\n");
		printf("   -t\t= Set tone squelch (103.5, 141.3)\n");
		printf("   -a\t= Turn AGC on\n");
		printf("   -n\t= Turn the noise blanker on\n");
		printf("   -r\t= Turn the RF attenuator on\n");
		printf("   -s\t= Turn on scanner mode\n");
		printf("Standard mode: \n");
		printf("   freq\t = Frequency in megahertz\n");
		printf("Example:\n\tpcr -p/dev/ttyS0 -v70 -q40 -f15 -mnfm 146.52\n");
		printf("Scanner mode: \n");
		printf("   -h\t= Set hold point (0 to 255)\n");
		printf("   lower_freq\t = Lower frequency in megahertz\n");
		printf("   upper_freq\t = Upper frequency in megahertz\n");
		printf("   step_freq\t = Step frequency in kilohertz\n");
		printf("Example:\n\tpcr -p/dev/ttyS0 -v70 -q40 -f15 -mnfm -s 144 148 5\n"); 
}

pcrfreq_t freqconv(char *strfreq)
{
	pcrfreq_t tempfreq;
	char *mhz, *khz;
	int i;

	mhz = strfreq;
	khz = strchr(strfreq, '.');

	if (khz == NULL) {
		if (strlen(mhz) > 4) {
			fprintf(stderr, "Malformed frequency: %s\n", strfreq);
			return 0;
       	}
       	tempfreq = atoi(mhz) * 1000000;
       	return tempfreq;
   	}
   	else
   	*khz = '\0';
   	khz++;
	tempfreq = atoi(khz);

   	for (i = 5; i >= strlen(khz) ; i--)
        tempfreq *= 10;

   	tempfreq += atoi(mhz) * 1000000;

	return tempfreq;
}

pcrfreq_t stepconv(char *strstep)
{
	int step;
	char *khz, *hz;
	int i;

	khz = strstep;
	hz = strchr(strstep, '.');

	if (hz == NULL) {
		return atoi(khz) * 1000;
	}

	*hz = '\0';
	hz++;

	step = atoi(hz);

	for (i = 2; i >= strlen(hz); i--) {
		step *= 10;
	}

	step += atoi(khz) * 1000;

	return step;
}

const char *bargraph(pcrfreq_t lwrfreq, pcrfreq_t uprfreq, pcrfreq_t curfreq)
{
	int length;
	int i;
	int step;
	unsigned long int curkhz;
	char meter[80];
	char freq[20];
	
	bzero(meter, sizeof(meter));

	length = (int)((float)(curfreq - lwrfreq) / (float)(uprfreq - lwrfreq) * 30);
	for (i = 0; i < length; i++) {
		strcat(meter, "=");
	}
	strcat(meter, ">");

	for (i++; i <= 30; i++) {
		strcat(meter, ".");
	}

	sprintf(freq, " %i.%06.0i MHz", curfreq/1000000, curfreq - ((int)(curfreq/1000000))*1000000);

	strcat(meter, freq);
	return meter;
}
