#ifndef PCRDEF_H
#define PCRDEF_H
/* 
   Copyright (C) 1999 PolyWog and Javaman for Ghetto.Org
   This file is part of the PCR-1000 API Library.

   The PCR-1000 API Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The PCR-1000 API Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the PCR-1000 API Library; see the file LICENSE.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  
 */

/*
	This is the PCR-1000 Command Set define file. Basically this file
	consists of all of the pertinent command prefixes that are sent to
	the radio.
 */

typedef char pcrcmd_t;
typedef unsigned long int pcrfreq_t;

/* fundamental commands */
#define PCRQST		"\?"	   /* Suffix for Radio Query */
#define PCRECMD		"\x0d\x0a" /* Suffix for execute command */
/* NOTE: after issueing an init (one of two commands below) DO NOT
         read(). If you do, the read() will block and wont return.
         the radio doesn't return data after an initialization. You
	 must close the socket, and reopen it. You wont have to reopen
         the socket with wierd opts, unless you reset the socket to 
         the state as it was before . */
#define PCRINITM	"H101\x0d\x0aG300\x0d\x0a" /* init manual probe */
#define PCRINITA	"H101\x0d\x0aG301\x0d\x0a" /* init auto */

#define PCRSIG		"G3"	/* Signal Update */
#define PCRSIGOFF 	"G300"  /* program should poll status */
#define PCRSIGON	"G301"  /* radio sends status automagically when a change */
#define PCRSIGBOFF	"G302" 	/* binary mode off */
#define PCRSIGBON	"G303"	/* binary mode on  */

#define PCRPWR		"H1"	/* Power */
#define PCRPWROFF	"H100" 	/* power radio down */
#define PCRPWRON	"H101" 	/* power radio up   */

#define PCRVOL		"J40"	/* Volume */
#define PCRVOLON	"J4075" /* 75 % */
#define PCRVOLOFF	"J4000" /* Turns the volume all the way down */

#define PCRSQL		"J41"	/* Squelch */
#define PCRSQLO		"J4100"	/* Fully Open */
#define PCRSQLC		"J4145" /* closed (for most purposes) */

#define PCRIF		"J43"	/* IF Shift */
#define PCRIFC		"J4380"	/* IF Centered */

#define PCRAGC		"J45"	/* Automatic Gain Control */
#define PCRAGCOFF	"J4500"	/* AGC Off */
#define PCRAGCON	"J4501"	/* AGC On  */

#define PCRNB		"J46"	/* Noise Blanking */
#define PCRNBOFF	"J4600"	/* NB Off */
#define PCRNBON		"J4601"	/* NB On  */

#define PCRRFA		"J47"	/* RF Attenuator */
#define PCRRFAOFF	"J4700"	/* RFA Off */
#define PCRRFAON	"J4701" /* RFA On */

#define PCRVSC		"J50"	/* VSC */
#define PCRVSCOFF	"J5000"	/* VSC Off */
#define PCRVSCON	"J5001"	/* VSC On  */

#define PCRTSQL		"J51"	/* CTCSS - Tone Squelch */
#define PCRTSQLOFF	"J5100"	/* Tone Sql Off */
#define PCRTSQL67	"J5101" /* 67.0 Hz */
#define PCRTSQL69	"J5102" /* 69.3 Hz */
#define PCRTSQL710      "J5103" /* 71.0 */
#define PCRTSQL719	"J5104"	/* 71.9 Hz */
#define PCRTSQL74	"J5105"	/* 74.4 Hz */
#define PCRTSQL77	"J5106" /* 77.0 Hz */
#define PCRTSQL79	"J5107"	/* 79.7 Hz */
#define PCRTSQL82	"J5108"	/* 82.5 */
#define PCRTSQL85	"J5109"	/* 85.4 */
#define PCRTSQL88	"J5110"	/* 88.5 */
#define PCRTSQL91	"J5111" /* 91.5 */
#define PCRTSQL94	"J5112"	/* 94.8 */
#define PCRTSQL97	"J5113" /* 97.4 */
#define PCRTSQL100	"J5114"	/* 100.0 */
#define PCRTSQL103	"J5115"	/* 103.5 */
#define PCRTSQL107	"J5116"	/* 107.2 */
#define PCRTSQL110	"J5117" /* 110.9 */
#define PCRTSQL114	"J5118" /* 114.8 */
#define PCRTSQL118	"J5119"	/* 118.8 */
#define PCRTSQL123	"J5120"	/* 123.0 */
#define PCRTSQL127	"J5121"	/* 127.3 */
#define PCRTSQL131	"J5122"	/* 131.8 */
#define PCRTSQL136	"J5123"	/* 136.5 */
#define PCRTSQL141	"J5124"	/* 141.3 */
#define PCRTSQL146	"J5125"	/* 146.2 */
#define PCRTSQL151	"J5126"	/* 151.4 */
#define PCRTSQL156	"J5127"	/* 156.7 */
#define PCRTSQL159	"J5128"	/* 159.8 */
#define PCRTSQL162	"J5129"	/* 162.2 */
#define PCRTSQL165	"J5130"	/* 165.5 */
#define PCRTSQL167	"J5131"	/* 167.9 */
#define PCRTSQL171	"J5132"	/* 171.3 */
#define PCRTSQL173	"J5133"	/* 173.8 */
#define PCRTSQL177	"J5134"	/* 177.3 */
#define PCRTSQL179	"J5135"	/* 179.9 */
#define PCRTSQL183	"J5136" /* 183.5 */
#define PCRTSQL186	"J5137" /* 186.2 */
#define PCRTSQL189	"J5138" /* 189.9 */
#define PCRTSQL192	"J5139" /* 192.8 */
#define PCRTSQL196	"J5140"	/* 196.6 */
#define PCRTSQL199	"J5141"	/* 199.5 */
#define PCRTSQL203	"J5142"	/* 203.5 */
#define PCRTSQL206	"J5143"	/* 206.5 */
#define PCRTSQL210	"J5144" /* 210.7 */
#define PCRTSQL218	"J5145" /* 218.1 */
#define PCRTSQL225	"J5146" /* 225.7 */
#define PCRTSQL229	"J5147"	/* 229.1 */
#define PCRTSQL233	"J5148"	/* 233.6 */
#define PCRTSQL241	"J5149"	/* 241.8 */
#define PCRTSQL250	"J5150"	/* 250.3 */
#define PCRTSQL254	"J5151" /* 254.1 */

#define PCRUNK01	"J4A"	/* Unknown - 1 */
#define PCRUNK02	"J4A80"	/* Unknown - 2 */

#define PCRTFLTR	"LD82"	/* Tracking filter */
#define PCRTFLTR00	"LD8200"/* automagic tracking filter */
#define PCRTFLTR01	"LD8201"/* manual tracking filter */

/* frequency assignment */
#define PCRFRQ		"K0"	/* freq. header */
#define MAXFRQLEN	10	/* freq. len. 10 bytes (padded) <GMMMKKKHHH> */
#define LOWERFRQ	50000		/* lower bounds for frequency 50 kHz */
#define UPPERFRQ	1300000000	/* upper bound for frequency 1.3 GHz */

#define PCRMODLSB	"00"	/* Lower sideband */
#define PCRMODUSB	"01"	/* Upper sideband */
#define PCRMODAM	"02"	/* Amplitude Modulated */
#define PCRMODCW	"03"	/* Continuous Mode */
#define PCRMODUNK	"04"	/* unknown mode -- unused */
#define PCRMODNFM	"05" 	/* Narrowband FM */
#define PCRMODWFM	"06"	/* Wideband FM */

#define PCRFLTR3	"00"	/* 3 kHz 	*/
#define PCRFLTR6	"01"	/* 6 kHz 	*/
#define PCRFLTR15	"02"	/* 15 kHz 	*/
#define PCRFLTR50	"03"	/* 50 kHz 	*/
#define PCRFLTR230	"04"	/* 230 kHz 	*/

/* Radio Queries */
#define PCRQSQL		"I0"	/* Query Squelch Setting */
#define PCRQRST		"I1"	/* Query Signal Strength */
#define PCRQOFST	"I2"	/* Query Frequency Offset */
#define PCRQDTMF	"I3"	/* Query presense of DTMF Tone */
#define PCRQWAREZ	"G4"	/* Query Firmware revision */
#define PCRQDSP		"GD"	/* Query Presense of DSP */
#define PCRQCTY		"GE"	/* Query country / region */

/* Standard Radio Replies */
#define PCRAOK		"G000"	/* Reply: Ok */
#define PCRABAD		"G001"	/* Reply: There was an error */
#define PCRADSP		PCRQDSP	/* DSP Header */
#define PCRADSPNO	"GD00"	/* Not present */
#define PCRADSPOK	"GD01"	/* Present */
#define	PCRASQL		PCRQSQL	/* Squelch Header */
#define PCRASQLCL	"04"	/* Sqlch Closed */
#define PCRASQLOPN	"07"	/* Sqlch Open */
#define PCRARST		PCRQRST	/* Signal Strength with have this header
				   plus 00-FF from weak to strong */
#define PCRAOFST	PCRQOFST/* Frequency offset Header
				   plus 00-7F from extreme (-) to near ctr OR
				   plus 81-FF from near ctr to extreme (+) */
#define PCRAOFSTCTR	"I280"	/* Frequency (offset) centered */

#define PCRADTMF	PCRQDTMF/* DTMF Header */
#define PCRADTMFNO	"I300"	/* DTMF Not Heard */
#define PCRADTMF0	"I310"	/* DTMF 0 */
#define PCRADTMF1	"I311"	/* DTMF 1 */
#define PCRADTMF2	"I312"	/* DTMF 2 */
#define PCRADTMF3	"I313"	/* DTMF 3 */
#define PCRADTMF4	"I314"	/* DTMF 4 */	
#define PCRADTMF5	"I315"	/* DTMF 5 */
#define PCRADTMF6	"I316"	/* DTMF 6 */
#define PCRADTMF7	"I317"	/* DTMF 7 */
#define PCRADTMF8	"I318"	/* DTMF 8 */
#define PCRADTMF9	"I319"	/* DTMF 9 */
#define PCRADTMFA	"I31A"	/* DTMF A */
#define PCRADTMFB	"I31B"	/* DTMF B */
#define PCRADTMFC	"I31C"	/* DTMF C */
#define PCRADTMFD	"I31D"	/* DTMF D */
#define PCRADTMFS	"I31E"	/* DTMF * */
#define PCRADTMFP	"I31F"	/* DTMF # */

/* Radio miscellaneous functions */
#define PCRBD		"G1"	/* Baud Rate Header */
#define PCRBD300	"G100"	/* 300 baud */
#define PCRBD1200	"G101"	/* 1200 baud */
#define PCRBD2400	"G102"	/* 2400 baud */
#define PCRBD9600	"G103"	/* 9600 baud */
#define PCRBD19200	"G104"	/* 19200 baud */
#define PCRBD38400	"G105"	/* 38400 baud */

/* BandScope functions */
#define PCRSBSC		"ME00001"	/* bandscope prefix SENT */
#define PCRRBSC		"NE1"		/* bandscope prefix RECV */
#define PCRRBSC0	"NE100"		/* packet 0 */
#define PCRRBSC1	"NE110"		/* packet 1 */
#define PCRRBSC2	"NE120"		/* packet 2 */
#define PCRRBSC3	"NE130"		/* packet 3 */
#define PCRRBSC4	"NE140"		/* packet 4 */
#define PCRRBSC5	"NE150"		/* packet 5 */
#define PCRRBSC6	"NE160"		/* packet 6 */
#define PCRRBSC7	"NE170"		/* packet 7 */
#define PCRRBSC8	"NE180"		/* packet 8 */
#define PCRRBSC9	"NE190"		/* packet 9 */
#define PCRRBSCA	"NE1A0"		/* packet 10 */
#define PCRRBSCB	"NE1B0"		/* packet 11 */
#define PCRRBSCC	"NE1C0"		/* packet 12 */
#define PCRRBSCD	"NE1D0"		/* packet 13 */
#define PCRRBSCE	"NE1E0"		/* packet 14 */
#define PCRRBSCF	"NE1F0"		/* packet 15 */



#endif /* PCRDEF_H */
