/*
   Copyright (C) 1999 PolyWog and Javaman for Ghetto.Org
   This file is part of the PCR-1000 API Library.

   The PCR-1000 API Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The PCR-1000 API Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the PCR-1000 API Library; see the file LICENSE.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

/*
 *  This file declares PCP.
 *  PCP Is the actual object that interfaces with the GUI
 *  This API deals with the error handling and the calls
 *  that must be made to and from the radio, via the PComm
 *  serial i/o object.
 */

#include "pcp.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#if defined (SunOS) || defined (Irix)
#include <strings.h>
#else // def SunOs/Irix
#include <string.h>
#endif // def SunOs/Irix

PCP :: PCP(char *port, tcflag_t speed, const char *name)
{
    PCPRadio = new PRadInf;
    PCPComm = new PComm(port, speed, name);

    bzero(PCPRadio, sizeof( PRadInf));
    bzero(PCPTemp, sizeof(PCPTemp));

    strncpy(PCPRadio->PCPPort, port, 0xff);
    PCPRadio->PCPSpeed = speed;
    PCPRadio->PCPVolume = 0;
    PCPRadio->PCPSquelch = 0;
    PCPRadio->PCPFreq = 146000000;
    strncpy(PCPRadio->PCPMode, PCRMODNFM, sizeof(PCPRadio->PCPMode));
    strncpy(PCPRadio->PCPFilter, PCRFLTR15, sizeof(PCPRadio->PCPFilter));
    PCPRadio->PCPToneSq=0;
    PCPRadio->PCPToneSqFloat=0.0;
    PCPRadio->PCPAutoGain=false;
    PCPRadio->PCPNoiseBlank=false;
    PCPRadio->PCPRFAttenuator=false;
    PCPRadio->PCPAutoUpdate = false;
    PCPStatus = false;
}

PCP :: ~PCP()
{
    delete PCPComm;
}

bool PCP :: PCPInit(bool autoUpdate)
/*
    this function sends the initialsation command(s).
    Due to the hardware taking some time to initialize,
    we sleep for one second to let it catch up, then check
    to see if the radio was on. If it *was* on then 
    it checks for a response.
    after checkresponse tells it that the command
    succeeded, it sets PCPRadio->PCPAutoUpdate mode appropriately
    and returns true.
    if checkresponse fails. It returns false.
 */
{
    if (autoUpdate==false) {
        PCPComm->PCTell(PCRINITM);
	fprintf(stderr, "Radio is coming up. Please wait...\n");
	sleep(1); 
	if (PCPStatus) 
        if (PCPCheckResponse()) {
            PCPRadio->PCPAutoUpdate=false;
            return true;
        }
    } else {
        PCPComm->PCTell(PCRINITA);
	sleep(1);
	if (PCPStatus)
        if (PCPCheckResponse()) {
            PCPRadio->PCPAutoUpdate=true;
            return true;
        }
    }

    return false;
}

bool PCP :: PCPPowerUp()
/*
    turns the radio's receiver on.
    returns:
        true/false  - based on PCPCheckResponse()
 */
{
    PCPComm->PCTell(PCRPWRON);
    if (PCPCheckResponse()) {
//	PCPCheckResponse();
	PCPStatus=true;
	return true;
    } 
    return false;
}

bool PCP :: PCPPowerDown()
/*
    turns the radio's receiver off.
    returns:
        true/false  - based on PCPCheckResponse()
 */
{
    PCPComm->PCTell(PCRPWROFF);
    // if (PCPCheckResponse()) {
	PCPCheckResponse();
	PCPStatus=false;
	return true;
    // } 
    // return false;
}

bool PCP :: PCPSetSpeed(tcflag_t speed)
/*
    First we check to see if the baudrate is right,
    if not then we just quietly return false. Then
    we decode the speed and grab the right speed.
    Then we tell the radio to switch speeds and
    set baudrate on the port by destroying PCOMM
    and reinitiating it with the new baud setting

    HOW TO USE THIS FUNCTION:
	create the object (at last known baudrate).
	call init
	call power up
	call this function
	delete the object
	create the object with the new speed setting
 */
{
    if ( (B300<=speed)&&(speed<=B38400) ) {
	bzero(PCPRadio->PCPInitSpeed, sizeof(PCPRadio->PCPInitSpeed));
        switch (speed) {
                case B38400:
                        // you probably want to set the speed
                        // to fastest available, so let's put
                        // this here first
                        strncpy(PCPRadio->PCPInitSpeed, PCRBD38400,
                                sizeof(PCPRadio->PCPInitSpeed)-1);
                        break;
                case B19200:
                        strncpy(PCPRadio->PCPInitSpeed, PCRBD19200,
                                sizeof(PCPRadio->PCPInitSpeed)-1);
                        break;
                case B300:
                        strncpy(PCPRadio->PCPInitSpeed, PCRBD300,
                                sizeof(PCPRadio->PCPInitSpeed)-1);
                        break;
                case B1200:
                        strncpy(PCPRadio->PCPInitSpeed, PCRBD1200,
                                sizeof(PCPRadio->PCPInitSpeed)-1);
                        break;
                case B2400:
                        strncpy(PCPRadio->PCPInitSpeed, PCRBD2400,
                                sizeof(PCPRadio->PCPInitSpeed)-1);
                        break;
                default:
                        // if all else fails, we'll always
                        // have paris! ~=^)
                        strncpy(PCPRadio->PCPInitSpeed, PCRBD9600,
                                sizeof(PCPRadio->PCPInitSpeed)-1);
        }
	PCPComm->PCTell(PCPRadio->PCPInitSpeed);
        delete PCPComm;
        PCPComm = new PComm(PCPRadio->PCPPort, speed, "PCPComm_R");
// investigate possible responses, i dont think one is given.
//	PCPCheckResponse();
        	PCPRadio->PCPSpeed = speed;
        	return true;
    }

    return false;
}

bool PCP :: PCPSetPort(const char *port) 
/*
    sets port by closing the filedes and reopening it
    on the new port.
 */
{
    PCPComm->PCClose();
    return(PCPComm->PCOpen(port));
}

bool PCP :: PCPSetVolume(int volume) 
/*
    checks to make sure volume is between 0 && 99
    sets current radio volume

    sprintf converts (and combines) the cmd with
    the argument, such that the argument has a minimum field width
    of two chars. If the fiels is < 2 chars (ie: arg=5) then it
    pads the field with one zero.
 */
{
    if ((0<=volume)&&(volume<=99)) {
        bzero(PCPTemp, sizeof(PCPTemp));
        sprintf(PCPTemp, "%s%0.2d", PCRVOL, volume);
        PCPComm->PCTell(PCPTemp);
        if (PCPCheckResponse()) {
            PCPRadio->PCPVolume=volume;
            return true;
        }
    }

    return false;
}

bool PCP :: PCPSetSquelch(int squelch) 
/*
    checks to make sure squelch is between 0 && 99
    sets current radio squelch

    sprintf converts (and combines) the cmd with
    the argument, such that the argument has a minimum field width
    of two chars. If the fiels is < 2 chars (ie: arg=5) then it
    pads the field with one zero.
 */
{
    if ((0<=squelch)&&(squelch<=99)) {
        bzero(PCPTemp, sizeof(PCPTemp));
        sprintf(PCPTemp, "%s%0.2d", PCRSQL, squelch);
        PCPComm->PCTell(PCPTemp);
        if (PCPCheckResponse()) {
            PCPRadio->PCPSquelch=squelch;
            return true;
        }
    }

    return false;
}

bool PCP :: PCPSetFreq(pcrfreq_t freq)
/*
    check to see if the frequencies are within bounds.
    populate a string, with correctly padded 0's based
    on the frequency fed in.
    1) copy the header, 2) concat the newly converted/padded
    frequency, 3) concat mode, 4) concat filter, 5) concat
    suffix.
    send the command to the radio, if it checks out
    set this as the new frequency and return a success.
 */
{
    char freqConv[32];
    bzero(freqConv, sizeof(freqConv));

    if ((LOWERFRQ<=freq)&&(freq<=UPPERFRQ)) {
        bzero(PCPTemp, sizeof(PCPTemp));
        sprintf(freqConv, "%0.10lu", freq);
        strcpy(PCPTemp, PCRFRQ);
        strcat(PCPTemp, freqConv);
        strcat(PCPTemp, PCPRadio->PCPMode);
        strcat(PCPTemp, PCPRadio->PCPFilter);
        strcat(PCPTemp, "00");
        PCPComm->PCTell(PCPTemp);
        if (PCPCheckResponse()) {
            PCPRadio->PCPFreq=freq;
#ifdef DEBUG_VER_
	    fprintf(stderr, "PCP: PCPSETFREQ - Success\n");
#endif
            return true;
        }
    }
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSETFREQ - Failed\n");
#endif
    return false;
}

bool PCP :: PCPSetMode(const char *mode)
/* 
	the concept is the same as above except it accepts
	standard text for "USB"/"LSB" etc...
 */
{
    char freqConv[32];
    char temp[8];

    bzero(PCPTemp, sizeof(PCPTemp));
    sprintf(freqConv, "%0.10lu", PCPRadio->PCPFreq);
    strcpy(PCPTemp, PCRFRQ);
    strcat(PCPTemp, freqConv);

    if ( strncmp(mode, "AM", 2)==0 ) {
	strcat(PCPTemp, PCRMODAM);
	strcpy(temp, PCRMODAM);
    } else if ( strncmp(mode, "CW", 2)== 0 ) {
	strcat(PCPTemp, PCRMODCW);
	strcpy(temp, PCRMODCW);
    } else if ( strncmp(mode, "LSB", 3)==0 ) {
	strcat(PCPTemp, PCRMODLSB);
	strcpy(temp, PCRMODLSB);
    } else if ( strncmp(mode, "USB", 3)==0 ) {
	strcat(PCPTemp, PCRMODUSB);
	strcpy(temp, PCRMODUSB);
    } else if ( strncmp(mode, "NFM", 3)==0 ) {
	strcat(PCPTemp, PCRMODNFM);
	strcpy(temp, PCRMODNFM);
    } else if ( strncmp(mode, "WFM", 3)==0 ) {
	strcat(PCPTemp, PCRMODWFM);
	strcpy(temp, PCRMODWFM);
    } else {
	return false;
    }

    strcat(PCPTemp, PCPRadio->PCPFilter);
    strcat(PCPTemp, "00");
    PCPComm->PCTell(PCPTemp);
    if (PCPCheckResponse()) {
#ifdef DEBUG_VER_
	fprintf(stderr, "Storing PCPRadio->PCPMode: %s\n", temp);
#endif // DEBUG_VER_
        strcpy(PCPRadio->PCPMode,temp);
        return true;
    }
    
    return false;

}

bool PCP :: PCPSetFilter(const char *filter)
/* 
        the concept is the same as above except it accepts
        standard text for "USB"/"LSB" etc...
 */
{
    char freqConv[32];
    char temp[8];

    bzero(PCPTemp, sizeof(PCPTemp));
    sprintf(freqConv, "%0.10lu", PCPRadio->PCPFreq);
    strcpy(PCPTemp, PCRFRQ);
    strcat(PCPTemp, freqConv);
    strcat(PCPTemp, PCPRadio->PCPMode);

    if ( strncmp(filter, "3", 1)==0 ) {
        strcat(PCPTemp, PCRFLTR3);
	strcpy(temp, PCRFLTR3);
    } else if ( strncmp(filter, "6", 1)== 0 ) {
        strcat(PCPTemp, PCRFLTR6);
	strcpy(temp, PCRFLTR6);
    } else if ( strncmp(filter, "15", 2)==0 ) {
        strcat(PCPTemp, PCRFLTR15);
	strcpy(temp, PCRFLTR15);
    } else if ( strncmp(filter, "50", 2)==0 ) {
        strcat(PCPTemp, PCRFLTR50);
	strcpy(temp, PCRFLTR50);
    } else if ( strncmp(filter, "230", 3)==0 ) {
        strcat(PCPTemp, PCRFLTR230);
	strcpy(temp, PCRFLTR230);
    } else {
        return false;
    }

    strcat(PCPTemp, "00");
    PCPComm->PCTell(PCPTemp);
    if (PCPCheckResponse()) {
#ifdef DEBUG_VER_
	fprintf(stderr, "Storing PCPRadio->PCPFilter: %s\n", temp);
#endif DEBUG_VER_
        strcpy(PCPRadio->PCPFilter,temp);
        return true;
    }

    return false;

}

bool PCP :: PCPSetToneSq(const char *value)
// set's the tone squelch for the radio. The default is
// value 00 for off. The values are NOT the hz, but the
// PCRDEF vals, 01=67.0 02=69.3 etc... if it checks out
// ok, then set this value, and return true.
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSETTONESQ (string) - %s\n", value);
#endif // DEBUG_VER_ 

	bzero(PCPTemp, sizeof(PCPTemp));
	strncpy(PCPTemp, PCRTSQL, sizeof(PCPTemp));
	strncat(PCPTemp, value, sizeof(PCPTemp));
	PCPComm->PCTell(PCPTemp);
	if (PCPCheckResponse()) {
		PCPRadio->PCPToneSq=value;
		return true;
	}

	return false;

}

bool PCP :: PCPSetToneSq(float passvalue)
// Since the previous method requires the programmer to
// remember the PCR-1000's internal number that corresponds
// to the tone squelch frequency, this overloaded method
// allows the programmer to pass a float, where the float
// is the frequency in question.
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSETTONESQ (float) - %f\n", passvalue);
#endif // DEBUG_VER_ 

	int tone;

	tone = (int)(passvalue * 10.0 + .1);  // Hack to remove truncating errors.
	PCPRadio->PCPToneSqFloat=passvalue;		

	switch (tone)
	{
		case 0:
			return PCPSetToneSq("00");
			break;
		case 670:
			return PCPSetToneSq("01");
			break;
		case 693:	
			return PCPSetToneSq("02");
			break;
		case 710:
			return PCPSetToneSq("03");
			break;
		case 719:
			return PCPSetToneSq("04");
			break;
		case 744:
			return PCPSetToneSq("05");
			break;
		case 770:
			return PCPSetToneSq("06");
			break;
		case 797:
			return PCPSetToneSq("07");
			break;
		case 825:
			return PCPSetToneSq("08");
			break;
		case 854:
			return PCPSetToneSq("09");
			break;
		case 885:
			return PCPSetToneSq("0A");
			break;
		case 915:
			return PCPSetToneSq("0B");
			break;
		case 948:
			return PCPSetToneSq("0C");
			break;
		case 974:
			return PCPSetToneSq("0D");
			break;
		case 1000:
			return PCPSetToneSq("0E");
			break;
		case 1035:
			return PCPSetToneSq("0F");
			break;
		case 1072:
			return PCPSetToneSq("10");
			break;
		case 1109:
			return PCPSetToneSq("11");
			break;
		case 1148:
			return PCPSetToneSq("12");
			break;
		case 1188:
			return PCPSetToneSq("13");
			break;
		case 1230:
			return PCPSetToneSq("14");
			break;
		case 1273:
			return PCPSetToneSq("15");
			break;
		case 1318:
			return PCPSetToneSq("16");
			break;
		case 1365:
			return PCPSetToneSq("17");
			break;
		case 1413:
			return PCPSetToneSq("18");
			break;
		case 1462:
			return PCPSetToneSq("19");
			break;
		case 1514:
			return PCPSetToneSq("1A");
			break;
		case 1567:
			return PCPSetToneSq("1B");
			break;
		case 1598:
			return PCPSetToneSq("1C");
			break;
		case 1622:
			return PCPSetToneSq("1D");
			break;
		case 1655:
			return PCPSetToneSq("1E");
			break;
		case 1679:
			return PCPSetToneSq("1F");
			break;
		case 1713:
			return PCPSetToneSq("20");
			break;
		case 1738:
			return PCPSetToneSq("21");
			break;
		case 1773:
			return PCPSetToneSq("22");
			break;
		case 1799:
			return PCPSetToneSq("23");
			break;
		case 1835:
			return PCPSetToneSq("24");
			break;
		case 1862:
			return PCPSetToneSq("25");
			break;
		case 1899:
			return PCPSetToneSq("26");
			break;
		case 1928:
			return PCPSetToneSq("27");
			break;
		case 1966:
			return PCPSetToneSq("28");
			break;
		case 1995:
			return PCPSetToneSq("29");
			break;
		case 2035:
			return PCPSetToneSq("2A");
			break;
		case 2065:
			return PCPSetToneSq("2B");
			break;
		case 2107:
			return PCPSetToneSq("2C");
			break;
		case 2181:
			return PCPSetToneSq("2D");
			break;
		case 2257:
			return PCPSetToneSq("2E");
			break;
		case 2291:
			return PCPSetToneSq("2F");
			break;
		case 2336:
			return PCPSetToneSq("30");
			break;
		case 2418:
			return PCPSetToneSq("31");
			break;
		case 2503:
			return PCPSetToneSq("32");
			break;
		case 2541:
			return PCPSetToneSq("33");
			break;
		default:
			PCPSetToneSq("00");
	}
	return false;
}

bool PCP :: PCPSetAutoGain(bool value)
// sets the automagic gain control to _value_
// (on/off) true/false... checks the radio response
// if ok, then sets the value and returns true, 
// otherwise returns false
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSETAUTOGAIN - %d\n", value);
#endif // DEBUG_VER_ 

	(value)?(PCPComm->PCTell(PCRAGCON)):(PCPComm->PCTell(PCRAGCOFF));

	if (PCPCheckResponse()) {
		PCPRadio->PCPAutoGain=value;
		return true;
	}

	return false;
}

bool PCP :: PCPSetNB(bool value)
// sets the noise blanking to _value_
// (on/off) true/false... checks the radio response
// if ok, then sets the value and returns true, 
// otherwise returns false
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSETNB - %d\n", value);
#endif // DEBUG_VER_ 

	(value)?(PCPComm->PCTell(PCRNBON)):(PCPComm->PCTell(PCRNBOFF));
	
	if (PCPCheckResponse()) {
		PCPRadio->PCPNoiseBlank=value;
		return true;
	}

	return false;
}

bool PCP :: PCPSetRFAttenuator ( bool value )
// sets the RF Attenuator to _value_
// (on/off) true/false... checks the radio response
// if ok, then sets the value and returns true, 
// otherwise returns false
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSETRFATTENUATOR - %d\n", value);
#endif // DEBUG_VER_ 

	(value)?(PCPComm->PCTell(PCRRFAON)):(PCPComm->PCTell(PCRRFAOFF));

	if (PCPCheckResponse()) {
		PCPRadio->PCPRFAttenuator=value;
		return true;
	}

	return false;
}

// getter methods

bool PCP :: PCPIsOn()
// returns true/false for radio on/off.
{
    return PCPStatus;
}

const char *PCP :: PCPSigStrengthStr()
// returns the radio's response to the signal strength
// query.  This includes the I1 header.  The rest of
// the data is the hexadecimal value.
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSIGSTRENGTH (string) Called\n");
#endif // DEBUG_VER_ 
	if(PCPComm->PCAsk(PCRQRST)) {
		if(PCPComm->PCHear(PCPTemp)) {
			return (const char*)PCPTemp;
		}
	}

	return 0;
}

int PCP :: PCPSigStrength()
// returns the signal strength as an integer between 0 and 255.
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPSIGSTRENGTH (int) Called\n");
#endif // DEBUG_VER_

	int sigstr;
	char digit;

	if(PCPComm->PCAsk(PCRQRST)) {
		if(PCPComm->PCHear(PCPTemp)) {
			digit = PCPTemp[2];
			if((digit >= 'A') && (digit <= 'F'))
				sigstr = (digit - 'A' + 1) * 16;
		 	else
				sigstr = atoi(&digit) * 16;

			digit = PCPTemp[3];
			if((digit >= 'A') && (digit <= 'F'))
				sigstr += digit - 'A' + 1;
			else
				sigstr += atoi(&digit);

			return sigstr;
		}
	}

	return NULL;
}

bool PCP :: PCPQueryOn()
// Querys radio for ONOFF actuator status,
// as opposed to checking the current state
// variable.
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PCP: PCPQUERYON Called\n");
#endif // DEBUG_VER_
	if(PCPComm->PCAsk("H1")) {
		if(PCPComm->PCHear(PCPTemp)) {
			if(strcmp(PCPTemp, "H101") == 0)
				return true;
			else
				return false;
		}
	}

	return false;
}

const char *PCP :: PCPGetPort()
// return a character string representing the current
// port setting /dev/*
{
    return PCPRadio->PCPPort;
}

tcflag_t PCP :: PCPGetSpeed_t()
// returns the tcflag_t version
{
	return PCPRadio->PCPSpeed;
}

const char *PCP :: PCPGetSpeed()
// returns a character string for the currently
// set port speed.
{
	switch(PCPRadio->PCPSpeed) {
		case B300:
			return "300";
			break;
		case B600:
			return "600";
			break;
		case B1200:
			return "1200";
			break;
		case B1800:
			return "1800";
			break;
		case B2400:
			return "2400";
			break;
		case B4800:
			return "4800";
			break;
		case B9600:
			return "9600";
			break;
		case B19200:
			return "19200";
			break;
		case B38400:
			return "38400";
			break;
		case B57600:
			return "57600";
			break;
		default:
			return "unknown";
			break;
	}
	return "unknown";
}

int PCP :: PCPGetVolume()
// returns an integer for the current volume setting
{
    return (PCPRadio->PCPVolume);
}

const char *PCP :: PCPGetVolumeStr()
// returns a const char string for the current volume setting
{
    sprintf(PCPTemp, "%d", PCPRadio->PCPVolume);
    return (PCPTemp);
}

int PCP :: PCPGetSquelch()
// returns an integer for the current squelch setting
{
    return (PCPRadio->PCPSquelch);
}

const char *PCP :: PCPGetSquelchStr()
// returns a const char string for the current squelch setting
{
    sprintf(PCPTemp, "%d", PCPRadio->PCPSquelch);
    return (PCPTemp);
}

pcrfreq_t PCP :: PCPGetFreq()
// returns an unsigned long int represnting current frequency
{
    return PCPRadio->PCPFreq;
}

const char *PCP :: PCPGetFreqStr()
// returns an const char string represnting current frequency
{
    sprintf(PCPTemp, "%0.10lu", PCPRadio->PCPFreq);
    return PCPTemp;
}

const pcrcmd_t *PCP :: PCPGetMode()
// returns a character string for the current mode
{
    return (PCPRadio->PCPMode);
}

const char *PCP :: PCPGetModeStr()
// returns a character string for the current mode
// converted to plain english
{
    bzero(PCPTemp, sizeof(PCPTemp));

    if (strcmp(PCRMODWFM, PCPRadio->PCPMode)==0) {
	strcpy(PCPTemp, "WFM");
	return PCPTemp;
    } else if (strcmp(PCRMODNFM, PCPRadio->PCPMode)==0) {
	strcpy(PCPTemp, "NFM");
	return PCPTemp;
    } else if (strcmp(PCRMODCW, PCPRadio->PCPMode)==0) {
	strcpy(PCPTemp, "CW");
	return PCPTemp;
    } else if (strcmp(PCRMODAM, PCPRadio->PCPMode)==0) {
	strcpy(PCPTemp, "AM");
	return PCPTemp;
    } else if (strcmp(PCRMODUSB, PCPRadio->PCPMode)==0) {
	strcpy(PCPTemp, "USB");
	return PCPTemp;
    } else if (strcmp(PCRMODLSB, PCPRadio->PCPMode)==0) {
	strcpy(PCPTemp, "LSB");
	return PCPTemp;
    } 
    
    strcpy(PCPTemp, "UNKNOWN");
    return PCPTemp;

}

const pcrcmd_t *PCP :: PCPGetFilter()
// returns a character string for the current filter
{
    return (PCPRadio->PCPFilter);
}

const char *PCP :: PCPGetFilterStr()
// returns a character string for the current filter
// converted to plain english
{
    bzero(PCPTemp, sizeof(PCPTemp));

    if (strcmp(PCRFLTR230, PCPRadio->PCPFilter)==0) {
	strcpy(PCPTemp, "230");
	return PCPTemp;
    } else if (strcmp(PCRFLTR50, PCPRadio->PCPFilter)==0) {
	strcpy(PCPTemp, "50");
	return PCPTemp;
    } else if (strcmp(PCRFLTR15, PCPRadio->PCPFilter)==0) {
	strcpy(PCPTemp, "15");
	return PCPTemp;
    } else if (strcmp(PCRFLTR6, PCPRadio->PCPFilter)==0) {
	strcpy(PCPTemp, "6");
	return PCPTemp;
    } else if (strcmp(PCRFLTR3, PCPRadio->PCPFilter)==0) {
	strcpy(PCPTemp, "3");
	return PCPTemp;
    } 

    return PCPRadio->PCPFilter;
}

const char *PCP :: PCPGetToneSq()
// returns the currently set tone squelch
{
    return PCPRadio->PCPToneSq;
}

const char *PCP :: PCPGetToneSqStr()
// returns a string representation of the last
// attempted tone squelch change.
{
    bzero(PCPTemp, sizeof(PCPTemp));
    sprintf(PCPTemp, "%f", PCPRadio->PCPToneSqFloat);
    return PCPTemp;
}

bool PCP :: PCPGetAutoGain()
// returns the currently set auto-gain value
{
    return PCPRadio->PCPAutoGain;
}	

const char *PCP :: PCPGetAutoGainStr()
// returns the currently set auto-gain string value
{
    bzero(PCPTemp, sizeof(PCPTemp));
    (PCPGetAutoGain())?(strcpy(PCPTemp,"1")):(strcpy(PCPTemp,"0"));
    return PCPTemp;
}

bool PCP :: PCPGetNB()
// returns the currently set noise-blanking value
{
    return PCPRadio->PCPNoiseBlank;
}

const char *PCP :: PCPGetNBStr()
// returns the currently set noise-blanking string value
{
    bzero(PCPTemp, sizeof(PCPTemp));
    (PCPGetNB())?(strcpy(PCPTemp,"1")):(strcpy(PCPTemp,"0"));
    return PCPTemp;
}


bool PCP :: PCPGetRFAttenuator()
// returns the currently set rf-attenuation value
{

    return PCPRadio->PCPRFAttenuator;
}

const char *PCP :: PCPGetRFAttenuatorStr()
// returns the currently set rf-attenuation string value
{
    bzero(PCPTemp, sizeof(PCPTemp));
    (PCPGetRFAttenuator())?(strcpy(PCPTemp,"1")):(strcpy(PCPTemp,"0"));
    return PCPTemp;
}


const struct PRadInf PCP :: PCPGetRadioInfo()
// returns current radio state. In case the person
// wants to save it etc...
{
    return *PCPRadio;
}

bool PCP :: PCPCheckResponse()
/*
    read from the radio for the PCRAOK and PCRABAD
    reply.
    returns:
        true    - for PCRAOK
        false   - for PCRABAD
        false   - and sets ErrRead to true if garbage was read.
 */
{
    PCPComm->PCHear(PCPTemp);
    if (strncmp(PCPTemp, PCRAOK, 4)==0) {
        PCPErrRead=false;
        return true;
    } else if (strncmp(PCPTemp, PCRABAD, 4)==0) {
        PCPErrRead=false;
        return false;
    }

    PCPErrRead = true;
    return false;
}

