/*
 *   QPcr1k 
 *   Copyright (C) 1999 by PolyWog and Javaman for Ghetto.Org
 *    <polywog@ghetto.org>, <javaman@ghetto.org>
 *
 *   A GUI front end using the QT Toolkit and PCP/PCOMM library
 *   for the Icom Pcr-1000 radio.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#if defined (SunOS) || (Irix)
#include <strings.h>
#else 
#include <string.h>
#endif

#include "pmainwindow.h"
#include "pfileio.h"
#include "prodlg.h"

#include <pcp.h>
#include <qstatusbar.h>
#include <qlabel.h>
#include <qframe.h>
#include <qmessagebox.h>
#include <qtimer.h>

#include "pui.h"
#include "pmenu.h"
#include "ptools.h"
#include "pstatus.h"
// #include "background.xpm"

PMainWindow :: PMainWindow (QWidget *parent, const char *name)
             : QMainWindow(parent, name)
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PMainWindow: Creating...\n");
#endif // DEBUG_VER_

    // create our main menubar and connect the dots... ;)
    // ok connect the signals emitted from PMMenuBar object
    // with the slots in this object
    PMMenuBar = new PMenu ( this, "menu" );
    connect (PMMenuBar, SIGNAL(newProfileCall()),   SLOT(PMNewProfile())   );
    connect (PMMenuBar, SIGNAL(loadProfileCall()),  SLOT(PMLoadProfile())  );
    connect (PMMenuBar, SIGNAL(saveasProfileCall()),SLOT(PMSaveAsProfile()));
    connect (PMMenuBar, SIGNAL(saveProfileCall()),  SLOT(PMSaveProfile())  );
    connect (PMMenuBar, SIGNAL(editProfileCall()),  SLOT(PMEditProfile())  );
    connect (PMMenuBar, SIGNAL(initRadioCall()),    SLOT(PMInit())         );
    connect (PMMenuBar, SIGNAL(powerRadioCall()),   SLOT(PMPower())        );
    connect (PMMenuBar, SIGNAL(aboutBoxCall()),     SLOT(PMAbout())        );
    connect (PMMenuBar, SIGNAL(exitProgramCall()),  SLOT(PMQuit())         );

    // create the status bar. This object will know that we are creating a
    // status bar because we said that *this* is our parent when creating
    // the statusbar. Hence we dont have to `add' anything. Since it is a
    // derivative of QStatusBar we overload the constructor so that our
    // text widgets are created and the *setter* methods (as slots) allow
    // access to each of the text widgets (which are actually QLabels).
    // -- see pstatus.h for more info.
    PMStatusBar = new PStatus(this, "pmstatusbar_pstatus");

    // create the tool bar... ptool will automatically insert
    // the toolbuttons and all that jazz.
    QString temp("Main Tool Bar");
    PMToolBar = new PTools(temp, this, this);

    // toolbar's been created. add it to *this* object.
    addToolBar(PMToolBar);

    // connect the toolbar's signals to their respective slots (note that
    // both above and here, we ommited the third argument to connect, which
    // *should* be the name of the object whose slot we are connecting to.
    // This is a neat (or not-so-neat) shortcut. Omitting the third object
    // assumes that we are talking about slots located in *this* object).
    connect (PMToolBar, SIGNAL(newProfileCall()),   SLOT(PMNewProfile())   );
    connect (PMToolBar, SIGNAL(loadProfileCall()),  SLOT(PMLoadProfile())  );
    connect (PMToolBar, SIGNAL(saveProfileCall()),  SLOT(PMSaveProfile())  );
    connect (PMToolBar, SIGNAL(editProfileCall()),  SLOT(PMEditProfile())  );
    connect (PMToolBar, SIGNAL(initRadioCall()),    SLOT(PMInit())         );
    connect (PMToolBar, SIGNAL(powerRadioCall()),   SLOT(PMPower())        );//
    connect (PMToolBar, SIGNAL(aboutBoxCall()),     SLOT(PMAbout())        );
    connect (PMToolBar, SIGNAL(exitProgramCall()),  SLOT(PMQuit())         );

    // create and add the new central widget. 
    PMInterface = new PUI (this, "pminterface_pui");
    setCentralWidget(PMInterface);

    // setup the defaults for the central widget,
    // first let's check for the file... if not, then set defaults.
    PMRcFile = new PFileIO();
    bzero(&PMRc, sizeof(PMRc));
    if (!PMRcFile->PFOpen(".qpcr1krc","r")) {
        // no rc file found... dont worry, we'll use these
        // defaults for now, and then create an rcfile
        // when we quit
        PMRadio = new PCP(PCPDefaultPort);
        PMRadio->PCPInit();
        PMRadio->PCPSetSpeed(B38400);
        delete PMRadio;
        PMRadio = new PCP(PCPDefaultPort, B38400);
        PMStatus=PMRadio->PCPQueryOn();
        if (!PMStatus) {
        	PMRadio->PCPPowerUp();
        }
        execRadioCommand("FREQUENCY", "0146000000");
        execRadioCommand("SQUELCH",   "00");
        execRadioCommand("VOLUME",    "00");
        execRadioCommand("MODE",      "NFM");
        execRadioCommand("FILTER",    "15");
        execRadioCommand("CTCSS",     "OFF");
        execRadioCommand("AGC",       "0");
        execRadioCommand("NBL",       "0");
        execRadioCommand("RFA",       "0");
    } else {
        // we have an rc file.
        if ( PMRcFile->PFRead(&PMRc, sizeof(PMRc)) ) {
            // tell the radio we want to change stuff
            // but only if we can read from the rc file ok.
            QString __RcLastCh;

            // create the radio device and configure it for the stored
            // speed. We must start at 9600 baud and then up it
            PMRadio = new PCP(PMRc.RcDevice);
            PMRadio->PCPInit();
            PMRadio->PCPSetSpeed(PMRc.RcSpeed);
            delete PMRadio;
            PMRadio = new PCP(PMRc.RcDevice, PMRc.RcSpeed);
            PMStatus = PMRadio->PCPQueryOn();
            if (!PMStatus) { PMRadio->PCPPowerUp(); }

            execRadioCommand("CHANNEL",   __RcLastCh.setNum(PMRc.RcLastCh));
            execRadioCommand("SQUELCH",   PMRc.RcSql);
            execRadioCommand("VOLUME",    PMRc.RcVol);
            execRadioCommand("AGC",       PMRc.RcAgc);
            execRadioCommand("NBL",       PMRc.RcNbl);
            execRadioCommand("RFA",       PMRc.RcRfa);
        }
    }
    delete PMRcFile;

    // set your tool bar information
    QString __gotport(PMRadio->PCPGetPort());
    QString __gotspeed(PMRadio->PCPGetSpeed());
    PMStatusBar->setDevice(__gotport);
    PMStatusBar->setPower(PMRadio->PCPQueryOn());
    PMStatusBar->setBaudRate(__gotspeed);

    // create a timer that will update the signal strength
    // this timer will trigger every 250 milliseconds, and is
    // set to not (false) be a single shot timer.
    PMStatTimer = new QTimer(this, "pmstattimer_qtimer");
    connect(PMStatTimer, SIGNAL(timeout()), SLOT(updateStatusSlot()));
    PMStatTimer->start(250, FALSE);
    
    // connect the signal emitted from the PUI object (which would be a 
    // radio call) to a local slot for the actual execution of the 
    // radio command
    connect(PMInterface, SIGNAL(execRadioCommand(const char*, const char*)),
		    SLOT(execRadioCommandSlot(const char*, const char*)) );


#ifdef DEBUG_VER_
	fprintf(stderr, "PMainWindow: Created\n");
#endif // DEBUG_VER_
}

PMainWindow :: ~PMainWindow()
{
    #ifdef DEBUG_VER_
    fprintf(stderr, "PMainWindow: Destroyed!\n");
    #endif // DEBUG_VER_

    PMRadio->PCPSetSpeed(B9600);
    delete PMRadio;
}

void PMainWindow :: execRadioCommandSlot(const char *name, const char *value)
// it calls a private function to execRadioCommand() becuase that way
// i can connect this as a slot, /and/ call the private function in a
// standard c++ call, hence reusing code.
{
#ifdef DEBUG_VER_
    fprintf(stderr, "PMainWindow: EXECRADIOCOMMANDSLOT - %s <-> %s\n", name, value);
#endif DEBUG_VER_
    execRadioCommand(name, value);

}

void PMainWindow :: updateStatusSlot()
{
	QString foo((const char*)PMRadio->PCPSigStrengthStr());
	QString foor(foo.right(2));
	PMStatusBar->setComm(foor);
	PMInterface->valueChangeSlot("SIGNAL", foo.right(2));
}

void PMainWindow :: PMAbout()
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PMainWindow: SLOT-PMABOUT\n");
#endif // DEBUG_VER_

    QMessageBox::message("QPcr1k for X-Windows",
			 "QPcr1k for X-Windows (C) 1999.\n"
			 "[by PolyWog and Javaman for Ghetto.Org]\n"
			 "GUI for the PCP/PCOMM API to control\n"
			 "the Icom PCR-1000 All-Band All-Mode \n"
			 "communications receiver.");
}

void PMainWindow :: PMNewProfile()
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PMainWindow: SLOT-PMNEWPROFILE\n");
#endif // DEBUG_VER_
}

void PMainWindow :: PMLoadProfile()
{
    #ifdef DEBUG_VER_
    fprintf(stderr, "PMainWindow: SLOT-PMLOADPROFILE\n");
    #endif // DEBUG_VER_
}

void PMainWindow :: PMSaveAsProfile()
{
    #ifdef DEBUG_VER_
    fprintf(stderr, "PMainWindow: SLOT-PMSAVEASPROFILE\n");
    #endif // DEBUG_VER_
}

void PMainWindow :: PMSaveProfile()
{
    #ifdef DEBUG_VER_
    fprintf(stderr, "PMainWindow: SLOT-PMSAVEPROFILE\n");
    #endif // DEBUG_VER_
}

void PMainWindow :: PMQuit()
// write the radio settings to a file before quitting. We want to save
// the current state of the receiver as the default VFO setting. In this
// case _RcLastCh_. The VFO will always be channel 0
{
    #ifdef DEBUG_VER_
    fprintf(stderr, "PMainWindow: SLOT-PMQUIT\n");
    #endif // DEBUG_VER_
    PMRcFile = new PFileIO();
    if (PMRcFile->PFOpen(".qpcr1krc","w")) {
        // if we can open this file for writing.
        bzero(&PMRc, sizeof(PMRc));             // this is always a safe thing to do
        strcpy (PMRc.RcMagic,   QPCR1K_VERSION); // our magic number ;)
        PMRc.RcLastCh = PMRc.RcCurrCh;          // the channel we were last at
        strncpy(PMRc.RcMem[0].ChFreq,           // current VFO data below goes into
                PMRadio->PCPGetFreqStr(),       // memory channel 0
                sizeof(PMRc.RcMem[0].ChFreq)-1   );
        strncpy(PMRc.RcMem[0].ChMode,
                PMRadio->PCPGetModeStr(),
                sizeof(PMRc.RcMem[0].ChMode)-1   );
        strncpy(PMRc.RcMem[0].ChFilter,
                PMRadio->PCPGetFilterStr(),
                sizeof(PMRc.RcMem[0].ChFilter)-1 );
        strncpy(PMRc.RcMem[0].ChToneSq,
                PMRadio->PCPGetToneSqStr(),
                sizeof(PMRc.RcMem[0].ChToneSq)-1 );
        strcpy (PMRc.RcMem[0].ChDesc, "VFO");   // a nifty description
        strncpy(PMRc.RcSql,     PMRadio->PCPGetSquelchStr(),     sizeof(PMRc.RcSql)-1    );
        strncpy(PMRc.RcVol,     PMRadio->PCPGetVolumeStr(),      sizeof(PMRc.RcVol)-1    );
        strncpy(PMRc.RcAgc,     PMRadio->PCPGetAutoGainStr(),    sizeof(PMRc.RcAgc)-1    );
        strncpy(PMRc.RcNbl,     PMRadio->PCPGetNBStr(),          sizeof(PMRc.RcNbl)-1    );
        strncpy(PMRc.RcRfa,     PMRadio->PCPGetRFAttenuatorStr(),sizeof(PMRc.RcRfa)-1    );
        strncpy(PMRc.RcDevice,  PMRadio->PCPGetPort(),           sizeof(PMRc.RcDevice)-1 );
        PMRc.RcSpeed = PMRadio->PCPGetSpeed_t();

        // now that we have gathered _state_ information into our struct... and
        // assuming that all the channels were saved correctly by the profile
        // editor dialog box. We shall write the contents of memory to file.
        PMRcFile->PFWrite(&PMRc, sizeof(PMRc));
        PMRcFile->PFClose();
    }
    delete PMRcFile;

    if (PMRadio != NULL) {
    	PMRadio->PCPPowerDown();
	    delete PMRadio;
    }
    qApp->quit();
}

void PMainWindow :: PMEditProfile()
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PMainWindow: SLOT-PMEDITPROFILE\n");
#endif // DEBUG_VER_
    // create the profile dialog box now, so we can save some time later.
    // make _this_ the parent, send it it's internal name, and make it MODAL
    ProDlg PMProfileDlg (&PMRc, this, "pmprofiledlg_prodlg", true);

    if ( PMProfileDlg.exec() ) {
        fprintf(stderr, "PMainWindow: PMProfileDlg->accept();\n");
    } else {
        fprintf(stderr, "PMainWindow: PMProfileDlg->reject();\n");
    }


}

void PMainWindow :: PMInit()
// send a radio initialization command to the unit
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PMainWindow: SLOT-PMINIT\n");
#endif // DEBUG_VER_

	// send the radio initialization command	
	PMRadio->PCPInit();

}

void PMainWindow :: PMPower()
{
#ifdef DEBUG_VER_
	fprintf(stderr, "PMainWindow: SLOT-PMPOWER\n");
#endif // DEBUG_VER_

	// send the radio power on command
	PMStatus=PMRadio->PCPIsOn();

	if (PMStatus) { 
		PMRadio->PCPPowerDown();
		PMStatus=false;
		PMStatTimer->stop();
	} else {
		PMRadio->PCPPowerUp();
		PMStatus=true;
		PMStatTimer->start(1*1000, FALSE);
	}
	PMStatusBar->setPower(PMStatus);
	
}

bool PMainWindow :: execRadioCommand(const char *name, const char *value)
// parses the command coming in and makes a call to the PCP
// object for the correct radio call. We will only call for a
// value change in the display if the exec'd command for the
// radio returns an ``ok'' code
{
    if (value==NULL) {
        // make sure that we WERE passed a value
        return false;
    }

    QString temp(value);
    int tempCh=0;
    bool val=false;

    if (strncmp(name, "VOLUME", 3)==0) {
        if (!PMRadio->PCPSetVolume(temp.toInt()))
          return false;
    } else if (strncmp(name, "SQUELCH", 3)==0) {
        if (!PMRadio->PCPSetSquelch(temp.toInt()))
          return false;
    } else if (strncmp(name, "FREQUENCY", 3)==0) {
        if (!PMRadio->PCPSetFreq(temp.toLong())) { return false; }
        strncpy(PMRc.RcMem[0].ChFreq,
                PMRadio->PCPGetFreqStr(),
                sizeof(PMRc.RcMem[0].ChFreq)-1   );
    } else if (strncmp(name, "MODE", 3)==0) {
        if (!PMRadio->PCPSetMode(value)) { return false; }
        strncpy(PMRc.RcMem[0].ChMode,
                PMRadio->PCPGetModeStr(),
                sizeof(PMRc.RcMem[0].ChMode)-1   );
    } else if (strncmp(name, "FILTER", 3)==0) {
        if (!PMRadio->PCPSetFilter(value)) { return false; }
        strncpy(PMRc.RcMem[0].ChFilter,
                PMRadio->PCPGetFilterStr(),
                sizeof(PMRc.RcMem[0].ChFilter)-1 );
    } else if (strncmp(name, "CTCSS", 3)==0) {
        if (!PMRadio->PCPSetToneSq(temp.toFloat())) {  return false; }
        strncpy(PMRc.RcMem[0].ChToneSq,
                PMRadio->PCPGetToneSqStr(),
                sizeof(PMRc.RcMem[0].ChToneSq)-1 );
    } else if (strncmp(name, "CHANNEL", 3)==0) {
        // set the temp to current channel so that we may
        // increment, or decrement it... otherwise set it
        // to the value passed in.
        tempCh = PMRc.RcCurrCh;
        if ( strcmp(temp, "<") == 0 ) {
            tempCh--;
        } else if ( strcmp(temp, ">") == 0) {
            tempCh++;
        } else {
            tempCh = temp.toInt();
        }
        if ( (tempCh<40) && (tempCh>=0) && (tempCh<=PMRc.RcMaxCh) ) {
            // if it's a valid channel set it and execute
            PMRc.RcLastCh = PMRc.RcCurrCh;
            PMRc.RcCurrCh = tempCh;
            temp.setNum(tempCh);
            execRadioCommand("FREQUENCY", PMRc.RcMem[PMRc.RcCurrCh].ChFreq);
            execRadioCommand("MODE",      PMRc.RcMem[PMRc.RcCurrCh].ChMode);
            execRadioCommand("FILTER",    PMRc.RcMem[PMRc.RcCurrCh].ChFilter);
            execRadioCommand("CTCSS",     PMRc.RcMem[PMRc.RcCurrCh].ChToneSq);
            PMInterface->valueChangeSlot(name, (const char*)temp);
            return true;
        } else {
            // otherwise do nothing and return false
            return false;
        }
    } else if (strncmp(name, "AGC", 3)==0) {
        (temp.toInt()==1)?(val=true):(val=false);
        if (!PMRadio->PCPSetAutoGain(val))
          return false;
    } else if (strncmp(name, "NBL", 3)==0) {
        (temp.toInt()==1)?(val=true):(val=false);
        if (!PMRadio->PCPSetNB(val))
          return false;
    } else if (strncmp(name, "RFA", 3)==0) {
        (temp.toInt()==1)?(val=true):(val=false);
        if (!PMRadio->PCPSetRFAttenuator(val))
          return false;
    } else {
        fprintf(stderr, "Unknown Command: %s\n", value);
        return false;
    }

    PMInterface->valueChangeSlot(name, value);
    return true;
}
